#!/bin/bash
fichierConfIpsec=/etc/freeswan/ipsec.conf
motif="conn"
commandeUp="/usr/sbin/ipsec auto --asynchronous --up"
commandeDown="/usr/sbin/ipsec auto --down"
commandeSupp="/usr/sbin/ipsec auto --delete"

function connexionsUp ()
{
	for connexion in `grep -w $motif $fichierConfIpsec | grep -F $1 | cut -d" " -f2`
	do
		$commandeUp $connexion
		if [ $? -eq 0 ]
		then
			echo "$connexion correctement activée."
		else
			echo -e "PROBLEME: IMPOSSIBLE D'ACTIVER $connexion\n"
		fi
		etatConnexions $1 
	done
}

function connexionsDown ()
{
	for connexion in `grep -w $motif $fichierConfIpsec | grep -F $1 | cut -d" " -f2`
	do
		echo $connexion
		$commandeDown $connexion
		if [ $? -eq 0 ]
		then
			echo "$connexion correctement désactivée."
		else
			echo -e "PROBLEME: IMPOSSIBLE DE DESACTIVER $connexion\n"
		fi
	done
}

function connexionsSupp ()
{
        for connexion in `grep -w $motif $fichierConfIpsec | grep -F $1 | cut -d" " -f2`
	do
		$commandeSupp $connexion
		if [ $? -eq 0 ]
		then
			echo "$connexion correctement supprimée."
		else
			echo -e "PROBLEME: IMPOSSIBLE DE SUPPRIMER $connexion\n"
		fi
	done
}

function etatConnexions ()
{
	echo "Etats des tunnels pour AMON $1:"
	for ipReseau in `grep -w $1 -A3 $fichierConfIpsec | grep -w leftsubnet | cut -d"=" -f2 | cut -d"/" -f1 | sort -u`
	do
		#ipPubliqueAmon=`grep -w $motif -A2 $fichierConfIpsec | grep -m1 "left=" | cut -d "=" -f2`
		#ipsec eroute | grep $ipReseau | grep "=>" #DEPRECATED#
		/usr/sbin/ipsec status | grep -F $ipReseau
	done
}

function afficherMsgErreur()
{
	echo -e "\nOptions possibles:\n"
	echo -e "\t -a|--activer CNDuAmon\t\t Pour activer les vpn d'un Amon\n"
       	echo -e "\t -d|--desactiver CNDuAmon\t Pour désactiver les vpn d'un Amon\n"
	echo -e "\t -e|--etat CNDuAmon\t\t Pour visualiser l'état les vpn d'un Amon\n"
	echo -e "\t -r|--recharger CNDuAmon\t Pour désactiver puis activer tous les vpn d'un Amon\n"
        echo -e "\t -s|--supprimer CNDuAmon\t Pour supprimer tous les vpn d'un Amon\n"
}

if [ $2 ]
then
	idAmon=$2
	if grep -w $idAmon $fichierConfIpsec > /dev/null 
	then
		case $1 in
			-a | --activer)
				#Fonction qui active sur le sphynx les connexions que l'on vient de créer
				connexionsUp $idAmon
				;;
			-d | --desactiver)
				connexionsDown $idAmon
				;;
			-r | --recharger)
				connexionsDown $idAmon
				connexionsUp $idAmon
				;;
			-e | --etat)
				if [ $2 ]
				then
					etatConnexions $idAmon
				else
					echo "Il faut indiquer le CN du Amon dont vous voulez voir l'état des tunnels (gererConnexions --recharger 0639632X-01)."
				fi
				;;
			-s | --supprimer)
				connexionsSupp $idAmon
				;;
			*)
				echo -e "\nOption $1 inexistante."
				afficherMsgErreur
				;;
		esac	
	else
		echo "Pas de AMON avec le CN $idAmon."
	fi
else
	echo -e "\nIl faut indiquer une option suivie du CN d'un AMON déclaré sur le Sphynx."
	afficherMsgErreur
fi
