# -*- coding: iso-8859-1 -*-
"""
Automatisation de la cration d'un tablissement
"""
from os import path, system
import re

from contexte import *
from data_parser import ConfigParser, Untyped

# en mode clef, la taille de la clef est fixe
TAILLE_CLEF = '2048'

class Automatisation:

    def __init__(self, filename):
        """
        @param filename : fichier de donnes 
        """
        self.filename = filename
        try:
            path.isfile(filename)
        except:
            raise Exception, "Pb avec le fichier de transition"
        self.ctx = Contexte()
        # fichier temporaire pour la cration 
        self.tmp_filename = "tmp/creation_etab.txt"
        #
        self.id_etab = None

        # dcoupe du fichier
        self.split()
        # ajout d'un etab
        self.create_etab()
        # ajout d'une connexion
        self.add_connexion()

    def split(self):
        """
        dcoupe le fichier suivant les diffrentes 'sections'
        """
        fh = file(self.filename,'r')
        s = fh.read()
        # split sur une ligne comportant au moins quatre tirets
        regexp = "----+\s*"
        l = re.split(regexp, s)
        self.creation_etab = l[0]
        self.ajout_connexions = l[1:]

        
    def create_etab(self):
        """Cration de l'tablissement"""
        # rcupration de l'identifiant tablissement
        c = ConfigParser()
        c.readstring(self.creation_etab)
        self.id_etab = c['id'][0]

        # ecriture du fichier de transition
        fh = file("../" + self.tmp_filename, 'w')

        # FIXME gnration du cn
        # print c.get_dict()

        dct = c.get_dict()

        # rcupration du certificat,
        # nous l'enlevons du dictionnaire (il n'est plus utilis aprs)
        
        certif = dct['certif']
        self._get_certif(certif[0])
        del dct['certif']
        
        etab_cfg = EtabCFG(dct).format()
        # fh.write(self.creation_etab)
        fh.write(etab_cfg)
        fh.close()

        # rcupreration du mode du sphynx
        if self.ctx.mode == "normal":
            # mode clef
            # il faudrait pouvoir rcuper la taille de la clef
            cmd = self.ctx.prepath + "-a -f %s -k %s -n %s" % (self.tmp_filename, str(TAILLE_CLEF), str(self.id_etab))
            os.system(cmd)
        else:
            # mode certificat (normal)
            # FIXME rcuprer l'identifiant
            cmd = self.ctx.prepath + "-a -f %s -n %s" % (self.tmp_filename,  str(self.id_etab))
            os.system(cmd)

        # self._purge()

    def _get_certif(self, name):
        """
        @param name: nom long du certificat (path + filename)
        """
        # path :
        dirname = path.dirname(name)
        # openssl pkcs7 -in $PATH -print_certs | /usr/share/eole/ParsePEM.py -o $CHEMIN >/dev/null 2>&1
        cmd = """openssl pkcs7 -in %s -print_certs | /usr/share/eole/ParsePEM.py -o %s >/dev/null 2>&1""" % (name, dirname)
        # execution de la commande 
        system(cmd)


    def add_connexion(self):
        """Ajout des connexions
        
        dico des connexions : 
        {
        'nom':'test_nom',
        'left':'test_left',
        'leftnexthop':'test_leftnexthop',
        'leftsubnet':'test_leftsubnet',
        'leftsubnet_masque':'test_leftsubnet_masque',
        'left_nat':'test_left_nat',
        }

        """
        # il faut recharger le sphynx car un tablissement a t ajout depuis
        self.ctx = Contexte()
        # mise des informations de connexion dans des objets ConfigParser
        for s in self.ajout_connexions:
            c = ConfigParser()
            c.readstring(s)
            # format de dict adapt aux entites
            u = Untyped(c.get_dict())
            # ajout de la connexion
            id_connexion = self.ctx.etab_connexion_add(self.id_etab, u)
            # ajout d'une translation
            trs = u['translation']

            if type(trs) == list:
                for value in trs:
                    l = value.split(":")
                    src = l[0]
                    dest = l[1]
                    # reload
                    self.ctx = Contexte()

                    d = {"source":src, "destination":dest}
                    self.ctx.etab_connexion_translation_add(self.id_etab, id_connexion, d)

            if type(trs) == str:
                    l = trs.split(":")
                    src = l[0]
                    dest = l[1]
                    # reload
                    self.ctx = Contexte()
                    
                    d = {"source":src, "destination":dest}
                    self.ctx.etab_connexion_translation_add(self.id_etab, id_connexion, d)

#     def _purge(self):
#         """Nettoyage"""
#         os.unlink( self.tmp_filename)


class EtabCFG:
    """
    Formatage d'un fichier etab.cfg avec les sections [administration] et [technique]
    """
    def __init__(self, dict):
        """
        @param am_dict: dictionnaie ambig
        """
        self.dict = dict
    
    def format(self):
        TEMPLATE = """
        [administration]
        nom = %s
        id = %s
        [technique]
        %s
        """
        return TEMPLATE % (self.dict['nom'][0], self.dict['id'][0], self._format_tech())

    
    def _format_tech(self):
        l = []
        for k in self.dict.keys():
            if k == "nom":
                continue
            if k == "id":
                continue
            # s'il y a plusieurs variables, elles sont toutes affiches
            for i in self.dict[k]:
                l.append("%s = %s" % (k, i))
                        
        return "\n".join(l)
