# -*- coding: iso-8859-1 -*-
"""
Contexte d'excution du sphynx pour
- le prompt
- l'interface en dialog curses
"""
import os, sys
import entities
from errors import *

class Contexte:
    """
    Classification de mthodes d'appel effectif au sphynx 
    """
    def __init__(self):
        """
        Instanciation du sphynx
        """
        try:
            # FIXME : passer le path en paramtres de contexte
            # DEBUG MODE (beurk)
            # path = "/home/gwen/home/projets/sphynx-vpn/"
            path = "/usr/share/eole/sphynx-vpn/"
            sphynx_file = path + "xml/sphynx.xml"
            cfg_file = path + "cfg/sphynx.cfg"

            # rglages pour les commandes du sphynx en perl
            # (le prepath est utilis par le prompt)
            self.prepath = "cd .. ; ./sphynx "
            # if os.path.isfile("../xml/sphynx.xml"):
            if os.path.isfile(sphynx_file):
                self.sphynx = entities.Sphynx(sphynx_file)
                # ok. le sphynx a t initialis
                # parseur de fichier sphynx.cfg
                try:
                    # fh = file("../cfg/sphynx.cfg",'r')
                    fh = file(cfg_file,'r')
                    string = fh.read()
                    if "normal" in string:
                        self.mode ="normal"
                    else:
                        self.mode = "pki"
                except:
                    raise SphynxError, \
                          "Le sphynx est initialise mais il n'y a pas de fichier de configuration"
        except Exception, e:
            raise Exception,e 
            sys.exit(1)

    def save(self):
        """Sauvegarde
        """
        self.sphynx.save_to_file()

    def reload(self):
        """
        Reconstitution de l'objet sphynx
        """
        self.sphynx = entities.Sphynx()

    def etab_connexion_add(self, id_etab, quest_dict, safe=False):
        """
        Ajout effectif d'une connexion
        @params quest_dict: le dictionnaire d'ajout de connexion
        """
        etab = self.sphynx.get_etab(id_etab)
        # concatenation du reseau et du masque
        if quest_dict.has_key('leftsubnet') and quest_dict.has_key('leftsubnet_masque'):
            trs = quest_dict['leftsubnet']
            del quest_dict['leftsubnet']
            quest_dict['leftsubnet']= trs + "/" + quest_dict['leftsubnet_masque']

        if safe:
            id_reseau = quest_dict['reseau']
            if id_reseau in etab.existing_networks():
                print "une connexion dans l'etablissement %s est deja renseignee avec le reseau %s " % (str(id_etab), str(id_reseau))
                print "=> aucune connexion n'a ete ajoutee dans cet etablissement"
                conn_added = "None"
            else:
                conn_added = etab.add_connexion(quest_dict)
                print "-- connexion ajoutee dans l'etab %s" % id_etab
                # sauvegarde des changements
                self.save()

        else:
            conn_added = etab.add_connexion(quest_dict)
            print "-- connexion ajoutee dans l'etab %s" % id_etab
            # sauvegarde des changements
            self.save()

        return conn_added
        
    def etab_connexion_delete(self, id_etab, id_connexion):
        """
        Suppression d'une connexion
        @param id_etab: identifiant de l'etablissement
        """
        etab = self.sphynx.get_etab(id_etab)
        etab.del_connexion(id_connexion)
        # sauvegarde des changements
        self.save()
        print "-- connexion supprimee dans l'etab %s" % id_etab

    def etab_get_version(self, id_etab):
        """
        param id_etab: l'identifiant de l'tablissement
	:return: la version de l'amon
        """
        etab = self.sphynx.get_etab(id_etab)
        return etab.get_version()

    def etab_connexion_list(self, id_etab):
        """
        Liste des connexions d'un etab : renvoie les noms des connexions
        @param id_etab: l'identifiant de l'tablissement
        @return: liste des connexions de l'etablissement
        """
        etab = self.sphynx.get_etab(id_etab)
        connexions = etab.get_connexions()
        return connexions

    def etab_connexion_translation_add(self, id_etab, id_connexion, trs_dict):
        """
        Ajout d'une translation  une connexion etablissement
        @param id_etab: identifiant de l'tablissement
        @param id_connexion: identifiant de la connexion
        """
        etab = self.sphynx.get_etab(id_etab)
        conn = etab.get_connexion(id_connexion)
        conn.add_translation(trs_dict['source'], trs_dict['destination'])
        # sauvegarde des changements
        self.save()
        print "-- translation ajoutee"

    def etab_connexion_translation_list(self, id_etab, id_connexion):
        """
        Liste des translations d'une connexion d'un etablissement
        @param id_etab: identifiant de l'tablissement
        @param id_connexion: identifiant de la connexion
        """
        etab = self.sphynx.get_etab(id_etab)
        conn = etab.get_connexion(id_connexion)
        trs_list = conn.translations
        return trs_list
        
    def etab_connexion_translation_delete(self, id_etab, id_connexion, trs_dict):
        """
        Suppression d'une translation  une connexion etablissement
        @param id_etab: identifiant de l'tablissement
        @param id_connexion: identifiant de la connexion
        @param trs_dict: source, destination de la translation
        """
        etab = self.sphynx.get_etab(id_etab)
        conn = etab.get_connexion(id_connexion)
        conn.del_translation(source=trs_dict['source'], destination=trs_dict['destination'])
        self.save()
        print "-- translation supprimee"
            
    def sphynx_reseau_add(self, quest_dict):
        """
        Ajout effectif du reseau
        @param quest_dict : dictionnaire des donnes reseau 
        """
        # concatenation du reseau et du masque
        if quest_dict.has_key('rightsubnet') and quest_dict.has_key('rightsubnet_masque'):
            quest_dict['rightsubnet']= quest_dict['rightsubnet'] + "/" + quest_dict['rightsubnet_masque']
        reseau_id = self.sphynx.acad.add_reseau(quest_dict)
        # sauvegarde des changements
        self.save()
        print "-- reseau <%s> ajoute" % reseau_id
        return reseau_id

    def sphynx_reseau_delete(self, id_reseau):
        """
        Suppression d'un reseau ct sphynx
        """
        self.sphynx.acad.del_reseau(id_reseau)
        self.save()
        print "-- reseau supprime"
        
    def sphynx_reseau_list(self):
        """
        Liste des reseaux cts academiques : renvoie les noms des rseaux
        """
        return self.sphynx.acad.get_reseaux()
