# -*- coding: iso-8859-1 -*-

"""

Parseur de fichier de donnes
-----------------------------

- some code borrowed from Guido's ``ConfigParser.py`` module

- some code borrowed from The Glyph in his early twisted's elucubrations

"""

import re

import types
import string

class Error(Exception):
    """Base class for ConfigParser exceptions."""

class ParsingError(Error):
    """Raised when a configuration file does not follow legal syntax."""


class ConfigParser:
    """Parseur de fichier de donnes sans redondance est sans section"""

    def __init__(self, ambiguous=True):
        """
        dict: le dictionnaire contenant les clef-valeurs
        """
        if ambiguous:
            self.dict = Ambiguous()
        else:
            self.dict = {}


    def read(self, filename):
        """Lecture d'un fichier d'options
        @param filename: nom du fichier de donnes
        """
        try:
            fp = open(filename)
        except IOError:
            raise ParsingError, "pb de lecture du fichier source"
        self._read(fp)
        fp.close()     


    def readlines(self, lines):
        """Lecture d'une liste de donnes
        @param lines: liste des lignes d'un fichier de donnes 
        """
        self._readlines(lines)

    def readstring(self, string):
        """Lecture  partir d'une string de donnes
        @params string: chane contenu d'un fichier de donnes
        """
        lines = string.split("\n")
        self._readlines(lines)

    def get(self, key):
        """Rcupre une option dans une section considre
        correspondant aux key/value options lines.
        """
        return self.dict[key]

    def items(self):
        """Rcupre tous les dicos d'une section"""
        return self.dict.items()

    def __getitem__(self, key):
        return self.dict.__getitem__(key)

    def get_dict(self):
        """renvoie le dictionnaire"""
        return self.dict
    
    # TODO : __setitem__, __delitem__

    OPTCRE = re.compile(
        r'(?P<option>[^:=\s][^:=]*)'          # very permissive!
        r'\s*(?P<vi>[:=])\s*'                 # any number of space/tab,
                                              # followed by separator
                                              # (either : or =), followed
                                              # by any # space/tab
        r'(?P<value>.*)$'                     # everything up to eol
        )

    def _read(self, fp):
        """Parse a option file"""
        self._readlines(fp.readlines())
    

    def _readlines(self, lines):
        """Traitement d'une liste de lignes"""

        optname = None
        e = None 
    
        for line in lines:

            # comment or blank line?
            if line.strip() == '' or line[0] in '#;':
                continue
            # continuation line?
            if line[0].isspace():
                continue
            # an option line?
            else:
                mo = self.OPTCRE.match(line)
                if mo:
                    optname, vi, optval = mo.group('option', 'vi', 'value')
                    if vi in ('=', ':') and ';' in optval:
                        # ';' is a comment delimiter only if it follows
                        # a spacing character
                        pos = optval.find(';')
                        if pos != -1 and optval[pos-1].isspace():
                            optval = optval[:pos]
                    optval = optval.strip()
                    # allow empty values
                    if optval == '""':
                        optval = ''
                    optname = self.optionxform(optname.rstrip())
                    self.dict[optname] = optval
                else:
                    # a non-fatal parsing error occurred.  set up the
                    # exception but keep going. the exception will be
                    # raised at the end of the file and will contain a
                    # list of all bogus lines
                    if not e:
                        e = ParsingError()
        
        # if any parsing errors occurred, raise an exception
        if e:
            raise e

    def optionxform(self, optionstr):
        """met les options systmatiquement en minuscule"""
        return optionstr.lower()


class Ambiguous:
    """A dictionary which may contain ambiguous entries.
    borrowed from the Glyph in his early twisted adventures, tanks to him ;)
    """
    def __init__(self):
        """Initialize.
        """
        self.data = {}

    def put(self, key,value):
        """Put a value into me with the given key.
        If I already have a reference to that key, further retrievals will be ambiguous.
        """
        if type(key)==types.StringType:
            key = string.lower(key)
        try:
            x = self.data[key]
            x.append(value)
        except:
            self.data[key] = [value]

        self.data = self.data

    __setitem__ = put
    
    def get(self,key):
        """Retrieve a value.
        
        This always returns a list; if it is empty, then I do not contain the
        given key.  If it has one entry, then the retrieval is not ambiguous.
        If it has more than one, the retrieval is ambiguous.
        """
        if type(key)==types.StringType:
            key = string.lower(key)
        x = self.data.get(key, [])
        return x

    __getitem__ = get

    def remove(self,key,value):
        """Remove one value for a particular key.
        If there are multiple entries under this key, disambiguate.  Otherwise,
        remove this key entirely.
        """
        if type(key)==types.StringType:
            key = string.lower(key)
        x = self.data[key]
        x.remove(value)
        if len(x)==0:
            del self.data[key]
        self.data = self.data


    def __delitem__(self, key): del self.data[key]
        
    def items(self):
        """
        Renvoie tous les couples 
        """
        return self.data

    def keys(self):
        return self.data.keys()

    def __repr__(self):
        return str(self.data)
    
    def has_key(self, key): return self.data.has_key(key)


    __str__ = __repr__

    

class Untyped:
    """
    Un dictionnaire qui contient  la fois des entres strictes et des entres ambigues.
    Un __getitem__ renvoie donc parfois une chane, parfois une liste.
    """
    def __init__(self, am):
        """
        @param am: instance de dictionnaire ambigu
        """
        self.orig = am
        
    def __getitem__(self, key):
        if len(self.orig[key]) != 1:
            # renvoie la liste elle-mme 
            return self.orig[key]
        else:
            # renvoie le premier lment de la liste
            return self.orig[key][0]

    def __setitem__(self, key, item):
        self.orig[key] = item

    def __delitem__(self, key): del self.orig[key]
    
    def items(self):
        # TODO : il faudrait ici aussi dtyper le dico 
        return self.orig.items()

    def keys(self):
        return self.orig.keys()

    def __delitem__(self, key): del self.orig[key]

    def has_key(self, key): return self.orig.has_key(key)
    
