# -*- coding: ISO-8859-1 -*-
"""

Ce module dfinit les entits dom  manipuler par l'objet Sphynx

"""

from xml.dom.minidom import parse

from helpers import *
from errors import *

"""
convention de configuration de la premire balise reseau_sphynx
"""
# nom du premier reseau_sphynx cr (pendant l'instanciation)
DEFAULT_NAME = "default"
# identifiant du premier reseau_sphynx
SPHYNX_ID = "sphynx-0"


class Sphynx:
    """
    Le sphynx dans son ensemble.
    Le node racine (DocumentNode)
    """
    def __init__(self, filepath = "../xml/sphynx.xml"):
        """
        @param filepath: path du fichier sphynx
        document : le document xml
        root_node : le node xml racine
        acad, etabs: les objets acad et etab
        """
        self.filepath = filepath
        # parsing du xml
        self.document, self.root_node = self._parse_file()
        self.acad, self.etabs = self._get_acad_etab()

    def _parse_file(self):
        """
        Parse un fichier sphynx
        """       
        # rcuprons le DOM document
        try:
            document = parse(self.filepath)
        except:
            raise SphynxError, "Pb avec le fichier sphynx"
        return document, document.documentElement

    def _get_acad_etab(self):
        """
        Parsing des noeuds dom
        """
        # noeud academique
        acad_node = self.root_node.getElementsByTagName('academie')[0]
        # creation de l'objet acad
        acad = Acad(acad_node)

        # rcupration des diffrents noeuds d'etab
        etab_list = self.root_node.getElementsByTagName('etablissement')
        etabs = []
        for etab in etab_list:
            # cration des objets etab
            etabs.append(Etab(etab, acad_node))

        return acad, etabs

    def save_to_file(self,filepath=None):
        """
        Enregistrement dans un fichier
        @param filepath: nom long du fichier  enregistrer
        """
        if filepath:
            filepath = filepath
        else:
            filepath = self.filepath
        fh = file(filepath, "w")
        # xmlstr = self.document.toxml(encoding='utf-8')
        #self.document.writexml(fh, encoding='utf-8')
        self.document.writexml(fh)
        # fh.write(xmlstr) 
        fh.close()
        
    def _toxml(self):
        """
        for debug use only
        """
        print self.document.toxml()

    def __repr__(self):
        return "<Sphynx -- >"

    def get_etab_list(self):
        """
        @return: liste des etablissement classs par rne
        permet de slectionner un tablissement et manipuler des connexions
        """
        try:
            l = []
            for etab in self.etabs:
                l.append(etab.get_id())
            return l
        except:
            raise EtabError, "la liste des etablissements est corrompue"
            
    def get_etab(self, id):
        """
        Renvoie l'objet Etab correspondant  l'id
        """
        for etab in self.etabs:
            if etab.get_id() == id:
                return etab
        raise EtabError, "identifiant etablissement errone"

class Etab:
    """
    Un Etablissement 
    """

    def __init__(self, node, ctx) :
        """
        @param node: node dom de l'tablissement
        @param ctx : le contexte est le contexte academique
        (nous avons besoin du node acadmique pour avoir la liste
        des reseau_sphynx)
        """
        self.node = node
        self.connexions = self._set_connexions()
        self.ctx = ctx
        # renseigne la connexion par dfaut 
        self.default = self._get_defaults()
        
    def _get_defaults(self):
        """
        Rcupration des donnes rseau par dfaut
        (normalement elles existent dans la premire balise reseau_sphynx)
        """
        try:
            conn = self._get_default_connexion()
            left = get_text(conn.node.getElementsByTagName('left')[0])
            left_nat = conn.node.getElementsByTagName('left')[0].getAttribute("nat")
            left_subnet = get_text(conn.node.getElementsByTagName('leftsubnet')[0])
            left_nexthop = get_text(conn.node.getElementsByTagName('leftnexthop')[0]) 
            return {"left":left,'left_nat':left_nat, "leftsubnet":left_subnet,"leftnexthop":left_nexthop}
        except:
            raise ConnexionError, "Probleme concernant la connexion par defaut"
        
    def _set_connexions(self):
        """
        connexions
        """
        connexions = []
        for conn in self.node.getElementsByTagName('connexion'):
            connexions.append(Connexion(conn))
        return connexions

    def get_version(self):
        """
        rcupre le type d'amon
        """
        return self.node.getAttribute("version")

    def set_version(self, version):
        """
        fixe le type d'amon
        version: 1.5, 2.0...
        """
        return self.node.setAttribute("version", version)

    def get_id(self):
        """
        rcupre l'identifiant (rne) de l'tablissement
        """
        return self.node.getAttribute("id")

    def get_nom(self):
        """
        rcupre le nom de l'tablissement 
        """
        return self.node.getAttribute("nom")

    def existing_networks(self):
        """renvoie la liste des reseaux existants
        dans toutes les connexions
        """
        return [conn.get_reseau() for conn in self.connexions]

    def set_identifiant_connexion(self):
        """
        Construction d'un identifiant de connexion
         partir
        - du RNE
        - du nombre le plus lev des identifiants existants
        """
        # return self.get_id() + '-' + str(len(self.connexions))
        return self.get_id() + '-' + str(next_id(self._get_ids()))

    def _valid_connexion(self, reseau):
        """
        Validation du reseau  l'ajout d'une connexion dans un etab
        vrifie que :
        - le reseau_sphynx existe
        """
        reseaux = []
        for res in self.ctx.getElementsByTagName('reseau_sphynx'):
            reseaux.append(str(ReseauSphynx(res).get_id()))

        if reseau  in reseaux:
            return True
        else:
            return False
        
    def add_connexion(self, dictionnaire) :
        """
        ajout d'un tunnel au node etablissement
        @param dico: de format suivant
        {'nom', 'left', 'left_nat', 'leftsubnet','leftnexthop'}
        """
        # conf par defaut
        dico = self.default
        # pas de left_nat par dfaut finalement ;)
        #del(dico['left_nat'])
        dico.update(dictionnaire)
        # constitution d'un identifiant
        dico['id'] = self.set_identifiant_connexion()
        # validation du reseau_sphynx
        # pas d'attribut reseau : c'est le reseau par defaut
        if not dico.has_key('reseau'):
            dico['reseau'] = SPHYNX_ID
        # s'il n'y a pas de nom, alors le nom et l'identifiant sont identiques
        if not dico.has_key('nom'):
            dico['nom'] = dico['id']
        else : 
            # - l'attribut propos n'est pas dans la liste
            if not self._valid_connexion(dico['reseau']):
                raise ConnexionError, "probleme lors de l'ajout d'une connexion : l'identifiant du reseau n'existe pas"

        # cration du node xml et ajout  la liste
        try:
            conn = lmx("connexion")
            conn["id"] = dico['id']
            conn["nom"] = dico['nom']
            conn["reseau"] = dico['reseau']
	    if dico.has_key('left_nat'):
                conn.left(nat=dico['left_nat']).text(dico['left'])
            else:
                # pas de left_nat par dfaut
                conn.left().text(dico['left'])
                
            conn.leftsubnet().text(dico['leftsubnet'])
            conn.leftnexthop().text(dico['leftnexthop'])
            # ajout de la connexion  la liste
            self.connexions.append(Connexion(conn.node))
            # ajout de la connexion au node etablissement
            self.node.appendChild(conn.node)
        except Exception, e:
            raise ConnexionError, "probleme lors de l'ajout d'une connexion"
        # renvoie l'identifiant de la connexion utilise 
        return conn["id"]

    def get_connexion(self,id):
        """
        Rcupre un node connexion (en vue d'une manipulation ou d'un effacement)
        """
        for conn in self.connexions:
            if conn.get_id() == id:
                return conn
        raise ConnexionError, "no such id"

    def _get_default_connexion(self):
        """
        Rcupre la connexion par dfaut de manire  renseigner la variable default 
        """
        # reconstitution de l'identifiant par dfaut
        identifiant = self.get_id() + "-0"
        try:
            conn = self.get_connexion(identifiant)
            return conn
        except:
            raise ConnexionError, "pas de connexion par defaut"

    def del_connexion(self, id) :
        """
        Suppression d'un tunnel au node etablissement 
        """
        # si la connexion est la connexion par defaut, interdire la suppression
        identifiant = self.get_id() + "-0"
        if id == identifiant:
            raise ConnexionError, "impossible de supprimer la connexion par defaut"
        else:
            try:
                conn = self.get_connexion(id)
                self.connexions.remove(conn)
                self.node.removeChild(conn.node)
            except:
                raise ConnexionError, "la connexion n'a pas pu tre supprimee"

    def get_connexions(self):
        """
        Liste id, noms des connexions d'un tablissement
        """
        l = []
        for conn in self.connexions:
            l.append([ conn.get_id(), conn.get_nom()])
        return l

    def _get_ids(self):
        """
        Liste des ids des connexions d'un etablissement
        """
        l = []
        for conn in self.connexions:
            l.append(conn.get_id())
        return l
        
    def __repr__(self) :
        return "<etablissement: -- %s>" % self.get_id()
    
class Connexion:
    """
    Une connexion ct etab
    """
    def __init__(self, node):
        self.node = node
        self.translations = self._set_translations()

    def get_id(self):
        """
        Identifiant de la connexion
        """
        return self.node.getAttribute("id")

    def set_id(self, attr):
        """
        Spcifie l'attribut identifiant de la connexion
        """
        self.node.setAttribute("id", attr)

    def get_nom(self):
        """
        rcupre le nom de la connexion
        """
        return self.node.getAttribute("nom")
    
    def get_translations(self):
        """
        Liste des translations 
        """
        return self.translations

    def get_reseau(self):
        """
        Rcupre le reseau_sphynx
        """
        return self.node.getAttribute("reseau")

    def set_reseau(self, attr):
        """
        Spcifie l'attribut reseau de la connexion
        """
        self.node.setAttribute("reseau", attr)

    def _set_translations(self):
        """
        Parse et dfinit les translations
        """
        translation_node = self.node.getElementsByTagName("translation")
        l = []
        for node in translation_node:
            l.append({"source":node.getAttribute("source"),
                      "destination":node.getAttribute("destination")}
                     )
        return l

    def _del_translation_node(self, source, destination):
        """
        Efface une translation d'un node connexion
        """
        translation_node = self.node.getElementsByTagName("translation")
        l = []
        for node in translation_node:
            if node.getAttribute("source") == source and node.getAttribute("destination") == destination:
                self.node.removeChild(node)
                

    def add_translation(self, source=None, destination=None):
        """
        Ajoute une translation pour la connexion en cours
        """
        try:
            assert source is not None
            assert destination is not None

            # cration du node xml
            trs = lmx("translation")
            trs["source"] = source
            trs["destination"] = destination

            # vrifions que la translation n'est pas dj prsente
            dico = {"source":trs['source'], "destination":trs['destination']}
            assert dico not in self.translations, "attention: cette translation existe deja"

            # ajout de la translation au node connexion
            self.node.appendChild(trs.node)

            # ajout  l'objet
            self.translations.append({"source":source, "destination":destination})

        except:
            raise ManipulationError, "probleme lors de l'ajout d'une translation"


    def del_translation(self, source=None, destination=None):
        """
        Suppression d'une translation
        """
        try:
            assert source is not None
            assert destination is not None
            assert {"source":source,"destination":destination} in self.translations, "attention: la translation a supprimer n'existe pas"
            
            # effacement du node
            self._del_translation_node(source, destination)
            # effacement dans l'objet
            self.translations.remove({"source":source,"destination":destination})

        except Exception, ValueError:
            raise ManipulationError, "la translation n'a pas pu tre supprimee"

    def __repr__(self):
        return "<Connexion: -- %s >" % self.get_id()
    
class Acad:
    """
    Sphynx ct academique
    """

    def __init__(self, node) :
        """
        @param node: le node etablissement
        """
        self.node = node

        self.default = self._get_defaults()
        self.reseaux = self._set_reseaux()

    def get_defaults(self):
        return self._get_defaults()

    def _get_defaults(self):
        """
        Rcupration des donnes rseau par dfaut
        (normalement elles existent dans la premire balise reseau_sphynx)
        """
        right = get_text(self.node.getElementsByTagName('right')[0])
        right_subnet = get_text(self.node.getElementsByTagName('rightsubnet')[0])
        right_nexthop = get_text(self.node.getElementsByTagName('rightnexthop')[0]) 
        try:
            right_redondee = get_text(self.node.getElementsByTagName('right_redondee')[0])
        except:
            return {"right":right,"rightsubnet":right_subnet,"rightnexthop":right_nexthop}
        else:
            return {"right":right,"rightsubnet":right_subnet,"rightnexthop":right_nexthop, "right_redondee":right_redondee}
    
    def _set_reseaux(self):
        """
        rcupration des ventuels rseaux sphynx
        """
        reseaux = []
        for res in self.node.getElementsByTagName('reseau_sphynx'):
            reseaux.append(ReseauSphynx(res))
        return reseaux

    def _set_reseau_dict(self):
        """
        Construction du dictionnaire des reseau_sphynx
        """
        d = {}
        for res in self.reseaux:
            d[res.get_id()] = res.get_nom()
        return d
    
    def _get_ids(self):
        """
        Liste des ids des reseaux_sphynx
        """
        l = []
        for i in self.reseaux:
            l.append(i.get_id())
        return l

    def set_identifiant_reseau(self):
        """
        Constitution d'un identifiant de reseau  partir :
        - du nombre de reseaux
        """
        # return "sphynx-" + str(len(self.reseaux))
        return "sphynx" + '-' + str(next_id(self._get_ids()))

    def add_reseau(self, add_dict):
        """
        Ajoute un reseau_sphynx ct acad
        """
        # constitution ventuelle d'un identifiant
        if not add_dict.has_key('id'):
            add_dict['id'] = self.set_identifiant_reseau()
        # constitution ventuelle d'un nom
        if not add_dict.has_key('nom'):
            add_dict['nom'] = add_dict['id']

        # conf par defaut
        default_dict = self.default
        # conf mise  jour avec les donnes aditionnelles
        default_dict.update(add_dict)
        # cration du node xml
        res = lmx("reseau_sphynx")
        res["id"] = default_dict['id']
        res["nom"] = default_dict['nom']
        res.right().text(default_dict['right'])
        res.rightsubnet().text(default_dict['rightsubnet'])
        res.rightnexthop().text(default_dict['rightnexthop'])
        if default_dict.has_key('right_redondee'):
            res.right_redondee().text(default_dict['right_redondee'])
        # ajout du rseau  la liste
        self.reseaux.append(ReseauSphynx(res.node))
        # ajout de la connexion au node etablissement
        self.node.appendChild(res.node)
        return res["id"]

    def _get_reseau(self,id):
        """
        Rcupre un node reseau d'identifiant id (en vue d'une manipulation ou d'un effacement)
        """
        for res in self.reseaux:
            if res.get_id() == id:
                return res
        raise ReseauSphynxError, "no such id"

    def get_reseaux(self):
        """
        @return: liste des noms, ids des reseaux sphynx
        """
        l = []
        for res in self.reseaux:
            l.append([res.get_id(), res.get_nom()])
        return l

    def del_reseau(self, id) :
        """
        suppression d'un reseau academique
        """
        # si le reseau est le reseau par defaut, interdire la suppression
        if id == SPHYNX_ID:
            raise ReseauSphynxError, "impossible de supprimer le reseau par default"
        else:
            try:
                res = self._get_reseau(id)
                self.reseaux.remove(res)
                self.node.removeChild(res.node)
            except:
                raise ReseauSphynxError, "impossible de supprimer ce reseau academique" 

    def __repr__(self):
        return "<Acad: -- >"

class ReseauSphynx:
    """
    Balise reseau_sphynx ct acadmique 
    """
    def __init__(self, node):
        self.node = node

    def get_id(self):
        """
        Identifiant du rseau
        """
        return self.node.getAttribute("id")

    def get_nom(self):
        """
        Nom du rseau
        """
        return self.node.getAttribute("nom")

    def __repr__(self):
        return "<ReseauSphynx: -- %s >" % self.get_id()

if __name__ == "__main__":
    pass
