# -*- coding: ISO-8859-1 -*-
"""

Ce module dfinit les entits dom  manipuler par l'objet Sphynx
pour la mise  jour des sphynx

"""

from xml.dom.minidom import parse

from helpers import *
from errors import *

"""
convention de configuration de la premire balise reseau_sphynx
"""
# nom du premier reseau_sphynx cr (pendant l'instanciation)
DEFAULT_NAME = "default"
# identifiant du premier reseau_sphynx
SPHYNX_ID = "sphynx-0"


class Sphynx:
    """
    Le sphynx dans son ensemble.
    Le node racine (DocumentNode)
    """
    def __init__(self, filepath = "../xml/sphynx.xml"):
        """
        @param filepath: path du fichier sphynx
        document : le document xml
        root_node : le node xml racine
        acad, etabs: les objets acad et etab
        """
        self.filepath = filepath
        # parsing du xml
        self.document, self.root_node = self._parse_file()
        self.acad, self.etabs = self._get_acad_etab()

    def _parse_file(self):
        """
        Parse un fichier sphynx
        """       
        # rcuprons le DOM document
        try:
            document = parse(self.filepath)
        except:
            raise FileError, "Pb avec le fichier sphynx"
        return document, document.documentElement

    def _get_acad_etab(self):
        """
        Parsing des noeuds dom
        """
        # noeud academique
        acad_node = self.root_node.getElementsByTagName('academie')[0]
        # creation de l'objet acad
        acad = Acad(acad_node)

        # rcupration des diffrents noeuds d'etab
        etab_list = self.root_node.getElementsByTagName('etablissement')
        etabs = []
        for etab in etab_list:
            # cration des objets etab
            etabs.append(Etab(etab, acad_node))

        return acad, etabs

    def save_to_file(self,filepath=None):
        """
        Enregistrement dans un fichier
        @param filepath: nom long du fichier  enregistrer
        """
        if filepath:
            filepath = filepath
        else:
            filepath = self.filepath
        fh = file(filepath, "w")
        self.document.writexml(fh)
        # fh.write(self.document.toxml()) 
        fh.close()
        
    def _toxml(self):
        """
        for debug use only
        """
        print self.document.toxml()


    def __repr__(self):
        return "<Sphynx -- >"

    def get_etab_list(self):
        """
        @return: liste des etablissement classs par rne
        permet de slectionner un tablissement et manipuler des connexions
        """
        try:
            l = []
            for etab in self.etabs:
                l.append(etab.get_id())
            return l
        except:
            raise ManipulationError, "la liste des etablissements est corrompue"
            
    def get_etab(self, id):
        """
        Renvoie l'objet Etab correspondant  l'id
        """
        for etab in self.etabs:
            if etab.get_id() == id:
                return etab
        raise ManipulationError, "identifiant etablissement errone"


class Etab :
    """
    Un Etablissement 
    """

    def __init__(self, node, ctx) :
        """
        @param node: node dom de l'tablissement
        @param ctx : le contexte est le contexte academique
        (nous avons besoin du node acadmique pour avoir la liste
        des reseau_sphynx)
        """
        self.node = node
        self.connexions = self._set_connexions()
        self.ctx = ctx
        
    def _set_connexions(self):
        """
        connexions
        """
        connexions = []
        for conn in self.node.getElementsByTagName('connexion'):
            connexions.append(Connexion(conn))
        return connexions

    def get_id(self):
        """
        rcupre l'identifiant (rne) de l'tablissement
        """
        return self.node.getAttribute("id")

    def get_nom(self):
        """
        rcupre le nom de l'tablissement 
        """
        return self.node.getAttribute("nom")

    def set_identifiant_connexion(self):
        """
        Construction d'un identifiant de connexion
         partir
        - du RNE
        - du nombre de connexions existantes
        """
        return self.get_id() + '-' + str(len(self.connexions))

    def _valid_connexion(self, reseau):
        """
        Validation du reseau  l'ajout d'une connexion dans un etab
        vrifie que :
        - le reseau_sphynx existe
        - la connexion n'existe pas dj
        """
        reseaux = []
        for res in self.ctx.getElementsByTagName('reseau_sphynx'):
            reseaux.append(ReseauSphynx(res).get_id())

        if reseau  in reseaux:
            return True
        else:
            return False

    def add_connexion(self, dico) :
        """
        ajout d'un tunnel au node etablissement
        @param dico: de format suivant
        {'nom', 'left', 'left_nat', 'leftsubnet','leftnexthop'}
        """
        # constitution d'un identifiant
        dico['id'] = self.set_identifiant_connexion()

        # validation du reseau_sphynx
        # - pas d'attribut reseau : c'est le reseau par defaut
        # FIXME : fixer  un attribut de base en fait (le premier reseau_sphynx) 
        if not dico['reseau']:
            dico['reseau'] = ""
        else : 
            # - l'attribut propos n'est pas dans la liste
            if not self._valid_connexion(dico['reseau']):
                raise ManipulationError, "probleme lors de l'ajout d'une connexion : l'identifiant du reseau n'existe pas"

        # cration du node xml et ajout  la liste
        try:
            conn = lmx("connexion")
            conn["id"] = dico['id']
            conn["nom"] = dico['nom']
            conn["reseau"] = dico['reseau']
            conn.left(nat=dico['left_nat']).text(dico['left'])
            conn.leftsubnet().text(dico['leftsubnet'])
            conn.leftnexthop().text(dico['leftnexthop'])
            # ajout de la connexion  la liste
            self.connexions.append(Connexion(conn.node))
            # ajout de la connexion au node etablissement
            self.node.appendChild(conn.node)
        except:
            raise ManipulationError, "probleme lors de l'ajout d'une connexion"

    def get_connexion(self,id):
        """
        Rcupre un node connexion (en vue d'une manipulation ou d'un effacement)
        """
        for conn in self.connexions:
            if conn.get_id() == id:
                return conn
        raise Exception, "no such id"

    def del_connexion(self, id) :
        """
        Suppression d'un tunnel au node etablissement 
        """
        try:
            # mettre l'id systmatiquement en minuscules
            conn = self.get_connexion(id)
            self.connexions.remove(conn)
            self.node.removeChild(conn.node)
        except:
            raise ManipulationError, "la connexion n'a pas pu tre supprimee"

    def get_connexions(self):
        """
        Liste nom, id des connexions d'un tablissement
        """
        l = []
        for conn in self.connexions:
            l.append([conn.get_nom(), conn.get_id()])
        return l

    def __repr__(self) :
        return "<etablissement: -- %s>" % self.get_id()


def _get_defaults(acad):
    """
    DEPRECATED (ancien format)
    Rcupration des donnes rseau par dfaut
    """
    right = get_text(acad.node.getElementsByTagName('right')[0])
    right_subnet = get_text(acad.node.getElementsByTagName('rightsubnet')[0])
    right_nexthop = get_text(acad.node.getElementsByTagName('rightnexthop')[0]) 
    return {"right":right,"rightsubnet":right_subnet,"rightnexthop":right_nexthop}

def _delete_defaults(acad):
    """
    Suppression des donnes rseau par dfaut
    """
    right = acad.node.getElementsByTagName('right')[0]
    right_subnet = acad.node.getElementsByTagName('rightsubnet')[0]
    right_nexthop = acad.node.getElementsByTagName('rightnexthop')[0]
    acad.node.removeChild(right)
    acad.node.removeChild(right_subnet)
    acad.node.removeChild(right_nexthop)

def first_connexions(etabs):
    """
    - Ajoute un attribut reseau par dfaut  chaque connexion
     partir de l'identifiant de son tablissement
    - Normalise le nom de connexion  partir de l'identifiant etab.
    """
    for etab in etabs:
        rne = etab.get_id()
        conn = etab.connexions[0]
        identifiant = rne + "-0"
        conn.set_id(identifiant)
        conn.set_reseau(SPHYNX_ID)

def first_reseau_sphynx(acad):
    """
    Construit une balise reseau_sphynx pour l'acad
    et efface les balise qui renseignent ce rseau.

    L'identifiant est incrment : sphynx-N
    Le premier nom est statique : default
    @param acad : le node acadmique
    """
    defaults = _get_defaults(acad)
    defaults['id'] = SPHYNX_ID
    defaults['nom'] = DEFAULT_NAME
    acad.add_reseau(defaults)

    _delete_defaults(acad)

def set_sphynx_attribute(root):
    """
    Ajoute un attribut id au sphynx
    construit  partir du nom 
    """
    # calcul  partir du nom
    nom = root.getAttribute("nom")

    if "." in nom:
        nom = nom.split(".")[0]
    else:
        if "/CN=" in nom:
            nom = nom.split("/CN=")[1]

    identifiant = nom 
    root.setAttribute("id", identifiant)
    
class Connexion:
    """
    Une connexion ct etab
    """
    def __init__(self, node):
        self.node = node
        self.translations = self._set_translations()

    def get_id(self):
        """
        Identifiant de la connexion
        """
        return self.node.getAttribute("id")

    def set_id(self, attr):
        """
        Spcifie l'attribut identifiant de la connexion
        """
        self.node.setAttribute("id", attr)

    def get_nom(self):
        """
        rcupre le nom de la connexion
        """
        return self.node.getAttribute("nom")
    
    def get_translations(self):
        """
        Liste des translations 
        """
        return self.translations

    def get_reseau(self):
        """
        Rcupre le reseau_sphynx
        """
        return self.node.getAttribute("reseau")

    def set_reseau(self, attr):
        """
        Spcifie l'attribut reseau de la connexion
        """
        self.node.setAttribute("reseau", attr)

    def _set_translations(self):
        """
        Parse et dfinit les translations
        """
        translation_node = self.node.getElementsByTagName("translation")
        l = []
        for node in translation_node:
            l.append({"source":node.getAttribute("source"),
                      "destination":node.getAttribute("destination")}
                     )
        return l

    def _del_translation_node(self, source, destination):
        """
        Efface une translation d'un node connexion
        """
        translation_node = self.node.getElementsByTagName("translation")
        l = []
        for node in translation_node:
            if node.getAttribute("source") == source and node.getAttribute("destination") == destination:
                self.node.removeChild(node)
                

    def add_translation(self, source=None, destination=None):
        """
        Ajoute une translation pour la connexion en cours
        """
        try:
            assert source is not None
            assert destination is not None

            # cration du node xml
            trs = lmx("translation")
            trs["source"] = source
            trs["destination"] = destination

            # vrifions que la translation n'est pas dj prsente
            dico = {"source":trs['source'], "destination":trs['destination']}
            assert dico not in self.translations, "attention: cette translation existe deja"

            # ajout de la translation au node connexion
            self.node.appendChild(trs.node)

            # ajout  l'objet
            self.translations.append({"source":source, "destination":destination})

        except:
            raise ManipulationError, "probleme lors de l'ajout d'une translation"


    def del_translation(self, source=None, destination=None):
        """
        Suppression d'une translation
        """
        try:
            assert source is not None
            assert destination is not None
            assert {"source":source,"destination":destination} in self.translations, "attention: la translation a supprimer n'existe pas"
            
            # effacement du node
            self._del_translation_node(source, destination)
            # effacement dans l'objet
            self.translations.remove({"source":source,"destination":destination})

        except Exception, ValueError:
            raise ManipulationError, "la translation n'a pas pu tre supprimee"

    def __repr__(self):
        return "<Connexion: -- %s >" % self.get_id()
    
class Acad:
    """
    Sphynx ct academique
    """

    def __init__(self, node) :
        """
        @param node: le node etablissement
        """
        self.node = node

        self.default = self._get_defaults()
        self.reseaux = self._set_reseaux()

    def _get_defaults(self):
        """
        Rcupration des donnes rseau par dfaut
        (normalement elles existent dans la premire balise reseau_sphynx)
        """
        # TESTS
        right = get_text(self.node.getElementsByTagName('right')[0])
        right_subnet = get_text(self.node.getElementsByTagName('rightsubnet')[0])
        right_nexthop = get_text(self.node.getElementsByTagName('rightnexthop')[0]) 
        return {"right":right,"rightsubnet":right_subnet,"rightnexthop":right_nexthop}
    
    def _set_reseaux(self):
        """
        rcupration des ventuels rseaux sphynx
        """
        reseaux = []
        for res in self.node.getElementsByTagName('reseau_sphynx'):
            reseaux.append(ReseauSphynx(res))
        return reseaux

    def _set_reseau_dict(self):
        """
        Construction du dictionnaire des reseau_sphynx
        """
        d = {}
        for res in self.reseaux:
            d[res.get_id()] = res.get_nom()
        return d

    def get_identifiant_reseau(self):
        """
        Teste si l'id reseau existe,
        sinon, lve une exception.
        """
        try:
            identifiant = self.node.getAttribute("id")
            return identifiant
        except: 
            raise ManipulationError, "pas d'identifiant pour le sphynx..."
    
    def set_identifiant_reseau(self):
        """
        Constitution d'un identifiant de reseau  partir :
        - du nombre de reseaux
        """
        return "sphynx-" + str(len(self.reseaux))

    def add_reseau(self, add_dict):
        """
        Ajoute un reseau_sphynx ct acad
        """
        # constitution ventuelle d'un identifiant
        if not add_dict.has_key('id'):
            add_dict['id'] = self.set_identifiant_reseau()
        # constitution ventuelle d'un nom
        if not add_dict.has_key('nom'):
            add_dict['nom'] = add_dict['id']

        # conf par defaut
        default_dict = self.default
        # conf mise  jour avec les donnes aditionnelles
        default_dict.update(add_dict)
        # cration du node xml
        res = lmx("reseau_sphynx")
        res["id"] = default_dict['id']
        res["nom"] = default_dict['nom']
        res.right().text(default_dict['right'])
        res.rightsubnet().text(default_dict['rightsubnet'])
        res.rightnexthop().text(default_dict['rightnexthop'])

        # ajout du rseau  la liste
        self.reseaux.append(ReseauSphynx(res.node))
        # ajout de la connexion au node etablissement
        self.node.appendChild(res.node)
        

    def _get_reseau(self,id):
        """
        Rcupre un node reseau d'identifiant id (en vue d'une manipulation ou d'un effacement)
        """
        for res in self.reseaux:
            if res.get_id() == id:
                return res
        raise Exception, "no such id"

    def get_reseaux(self):
        """
        @return: liste des noms, ids des reseaux sphynx
        """
        l = []
        for res in self.reseaux:
            l.append([res.get_nom(), res.get_id()])
        return l

    def del_reseau(self, id) :
        """
        suppression d'un reseau academique
        """
        # si le reseau est le reseau par defaut, interdire la suppression
        if id == SPHYNX_ID:
            raise ManipulationError, "impossible de supprimer le reseau par default"
        else:
            try:
                res = self._get_reseau(id)
                self.reseaux.remove(res)
                self.node.removeChild(res.node)
            except:
                raise ManipulationError, "impossible de supprimer ce reseau academique" 

    def __repr__(self):
        return "<Acad: -- >"


class ReseauSphynx:
    """
    Balise reseau_sphynx ct acadmique 
    """
    def __init__(self, node):
        self.node = node

    def get_id(self):
        """
        Identifiant du rseau
        """
        return self.node.getAttribute("id")

    def get_nom(self):
        """
        Nom du rseau
        """
        return self.node.getAttribute("nom")

    def __repr__(self):
        return "<ReseauSphynx: -- %s >" % self.get_id()


if __name__ == "__main__":
    pass
