#!/usr/bin/env python
# -*- mode: Python; coding: iso-8859-1 -*-
"""
Moulinette de regeneration des conf de tous les etabs presents dans la base

"""

import sys
import commands
# entities_update se comporte diffremment
from entities_update import *
from helpers import *
from errors import *

import os
from glob import glob
from creole import parsedico

dico = parsedico.parse_dico()

from xml.dom.minidom import parse

def main():
    if not len(sys.argv) <= 2:
        USAGE = """
        Utilitaire de regeneration des fichiers de conf des etabs
        **parametre optionnel** : nom long du fichier cible
        """
        print USAGE
    else:
        if len(sys.argv) == 2:
            filename = sys.argv[1]
            main(filename)
        else:
            main()
            
def get_ids(filename="../xml/sphynx.xml"):
    try:
        s = Sphynx(filename)
        id_etabs= []
        for etabs in s.etabs:
            id_etabs.append(etabs.get_id())
    except Exception, e :
        print e.__class__ , ":", "Attention, probleme ..."


    return id_etabs

def get_id_sphynx(filename="../xml/sphynx.xml"):
    try:
        document = parse(filename)
        sphynx_node = document.getElementsByTagName('sphynx')[0]
        id_sphynx = sphynx_node.getAttribute('id')
        return id_sphynx
    except Exception, e :
        print e.__class__ , ":", "Attention, probleme ..."
    
# generation test-rvp
def gen_testrvp(id_etab, ip_sphynx, filename="test-rvp"):
    """
    generation fichier test-rvp
    recupere id_etab et adresse ip privee du sphynx 
    filename: nom fichier cible
    """
    template = """#!/bin/sh
# DEB:%s
echo -n "Reseau Virtuel Prive :"
if [  -e /var/lock/subsys/rvp ]
then
   fping %s
else
   echo "Le tunnel n'est pas actif!"
exit 2
fi
# FIN:%s
"""

    template = template % (id_etab, ip_sphynx, id_etab)

    fh = file(filename,'w')
    fh.write(template)
    fh.close()

# generation Id
def gen_Id(id_etab, filename="Id"):
    """ 
    generation du fichier Id 
    """
    fh = file(filename, 'w')
    fh.write(id_etab)
    fh.close()

# generation Recup-Crl.sh
def gen_RecupCrl(id_etab, url_crl, filename="Recup-Crl.sh"):
    """
    generation fichier Recup-Crl.sh
    recupere id_etab et url crls 
    filename: nom fichier cible
    """
    template = """#!/bin/sh
#----------------------------
# Recuperation
# des listes de revocations
#----------------------------
# Appel Bibliotheque
[ -x /usr/share/eole/FonctionsEole ] || {
	echo "Pas de bibliotheque Eole !"
	exit 1
}

. /usr/share/eole/FonctionsEole
ParseDico
VERSION=`cat /etc/eole/version`
rc=0

Zephir "INIT" "MAJ liste revocation"
[ "\$rc"  -ne '0' ] && {
	Zecho "Probleme Zephir"
	Zecho "RC = $rc"
	Zecho "Contacter l'administrateur"
	exit
}

RepCour=`pwd`
date >/tmp/RecupCrl.log
cd /etc/freeswan/ipsec.d/crls
# DEB:%s
wget --timestamping %s >> /tmp/RecupCrl.log 2>&1
# FIN:%s
/usr/sbin/ipsec auto --rereadcrls
cd $RepCour
Zephir "FIN" "MAJ liste revocation"
"""
    template = template % (id_etab, url_crl, id_etab)

    fh = file(filename,'w')
    fh.write(template)
    fh.close()
		    
# Regeneration de la conf
def gen_conf(id_etab):
    # on regnre la conf
    version = commands.getoutput("./tunnel --etab-get-version %s" % str(id_etab))
    #version = os.system("./tunnel --etab-get-version %s" % str(id_etab))
    version = str(version)
    # gestion des deux version (1.5 et ng)
    if "1.5" in version:
        os.system("../sphynx -g %s" % id_etab)
    else:    
    	os.system("../sphynx -z %s" % id_etab)

    # rpertoire cible
    if not os.path.isdir("/home/data/%s"% id_etab):
        os.system("mkdir /home/data/%s" % id_etab)
    # dplacement des fichiers de conf
    os.system("mv ../ipsec/*%s /home/data/%s"% (id_etab, id_etab))
    ## motif de filtrage avec le cn - id_etab - chiffre
    id_sphynx = get_id_sphynx()
    config_files = glob('../ipsec/ipsec_updown_%s-%s-[0-9]*'% (id_sphynx, id_etab))
    for conf in config_files:
        os.system("mv %s /home/data/%s"% (conf, id_etab))
    #generation fichier test-rvp
    filename="/home/data/%s/test-rvp" % id_etab
    if "2.0" in version:
        if dico['adresse_ip_redondee_eth1']:     
            gen_testrvp(id_etab, dico['adresse_ip_redondee_eth1'], filename=filename)
        else:
            gen_testrvp(id_etab, dico['adresse_ip_eth1'], filename=filename)
    #generation fichier Id
    filename="/home/data/%s/Id" % id_etab
    gen_Id(id_etab, filename=filename)
    if "1.5" in version:
        #generation fichier Recup-Crl.sh
        filename="/home/data/%s/Recup-Crl.sh" % id_etab
        # On regarde si on est en mode pki
        fh = file('../cfg/sphynx.cfg','r')
        mode = fh.read()
        fh.close()
        if "pki" in mode:
    	    url = dico['url_crl1']    
    	    if url.strip():
                gen_RecupCrl(id_etab, dico['url_crl1'], filename=filename)
    # mnage
    os.system("rm -f ../ipsec/*")
    

def gen_confs(etab_list):
    for id_etab in etab_list:
        gen_conf(id_etab)

def main(filename="../xml/sphynx.xml"):
    etab_list = get_ids(filename)
    gen_confs(etab_list)
        
if __name__ == "__main__":
    main()
    
