# -*- coding: iso-8859-1 -*-
"""

- Utilitaires de manipulation du xml dom standard

- Utilitaires pour la console (ou les questions au prompt)

"""
from xml.dom.minidom import *

from errors import *

def _parse_id(str):
    """
    Parsing d'un identifiant
    un identifiant est de la forme : [nom]-[numero]
    @return: numro de l'identifiant
    """
    l = str.split("-")
    return int(l[-1]) + 1

def next_id(list):
    """
    A partir d'une liste d'identifiant,
    rcupre le plus grand identifiant,
    @return: l'identifiant suivant
    """
    l = []
    # rcuprons une liste d'entier
    for i in list:
        l.append(_parse_id(i))
    # tri de la liste d'entiers
    l.sort()
    return l[-1]

def stringify(string):
    """
    Enlve le prfixe unicode dans la string
    Replacement des unicode u'' par rien (pour que l'affichage soit lisible)
    """
    string = str(string)
    return string.replace("u'","'")

def select_list(selection):
    """
    Utilitaire de construction d'une slection en ligne de commande
    @param selection : liste
    @return :  l'identifiant slectionn (entier)
    """
    # affichage de la liste (ordonne)
    for i in selection:
        print selection.index(i) , ':', stringify(i)
        # print selection.index(i) , ':', i[0]

    # recuperation du numero
    number = int(raw("Choisissez un numero dans la liste precedente"))
    return number

def raw(text):
    """
    Question en ligne de commande : permet de reprer si l'utilisateur a renvoy quelque chose
    
    @param text: le libell de message
    @return: la variable demande
    """
    var = raw_input(text + " : \n")
    if var:
        return var
    else:
        raise NoneException

def lance_prompt(questions, ordre):
    """
    Questions en ligne de commande :
    permet de stocker la srie des questions-rponses dans un dictionnaire
    
    @param dictionnaire: dictionnaire des variables et de leur libell
    """
    result = {}
    for var in ordre:
    # for var, libelle in questions.items():
        try:
            result[var] = raw(questions[var]) # raw(libelle)
        except NoneException:
            raise NoneException

    return result


def permissive_lance_prompt(questions, ordre):
    """
    Questions en ligne de commande :
    permet de stocker la srie des questions-rponses dans un dictionnaire
    (si une question est laisse vide, alors elle n'est pas prise en compte)
    @param dictionnaire: dictionnaire des variables et de leur libell
    """
    result = {}
    # for var, libelle in questions.items():
    for var in ordre:
        libelle = questions[var]
        result[var] = raw_input(libelle + " : \n")

    return _purge_dico(result)


def _purge_dico(dico):
    """
    Purge les valeurs vides d'un dico
    """
    for key, value in dico.items():
        if not value:
            dico.pop(key)

    return dico


"""
Utilitaires de sortie formate pour la console et la ligne de commande
"""

def couples_list(list):
    """
    Sortie d'une liste de liste de deux lments 
    """
    l = ["%s %s" % tuple(i) for i in list]
    return "\n".join(l)

def dict_list(list):
    """
    Sortie d'une liste de dictionnaires
    """
    l = []
    for d in list: 
        for i in d.keys():
            l.append("%s : %s  " % (i,d[i]))
        l.append("--")

    return "\n".join(l)

def dict_list_cmd(list):
    """
    Sortie d'une liste de dictionnaires
    """
    l = []
    for d in list: 
        l.append("%s  %s" % (d['source'],d['destination']))

    return "\n".join(l)



"""
Manipulations xml
=================

- ajout d'un node  un node courant : node.appendChild(add)

- substitution d'un node  un node courant : index_body.parentNode.replaceChild(matrix_body, index_body)

- suppression d'un node : a.parentNode.removeChild(a)

Recherche
=========

- recherche par noms de balise : node.getElementsByTagName(name)


"""

def get_text(node):
    """
    Renvoie la valeur textuelle d'un node simple (c'est--dire ne contenant que du texte)
    """
    include = ""
    for line in node.childNodes:
        include = include + line.data
    return include

#     text_node = node.childNodes[0]
#     return text_node.data

class lmx:
    """
    Creation facile de XML pour le minidom
    manipule ou cre un node dom

    node : le node dom  manipuler
    
    """

    def __init__(self, node='div'):
        if type(node) == str:
            node = Element(node)
        self.node = node

    def __getattr__(self, name):
        if name[0] == '_':
            raise AttributeError("no private attrs")
        return lambda **kw: self.add(name,**kw)

    def __setitem__(self, key, val):
        self.node.setAttribute(key, val)

    def __getitem__(self, key):
        return self.node.getAttribute(key)

    def text(self, txt):
        nn = Text()
        nn.data = txt
        self.node.appendChild(nn)
        return self

    def add(self, tagName, **kw):
        newNode = Element(tagName)
        self.node.appendChild(newNode)
        xf = lmx(newNode)
        for k, v in kw.items():
            if k[0] == '_':
                k = k[1:]
            xf[k]=v
        return xf


def doctest_node_manipulation():
    """
    >>> from xml.dom.minidom import *
    >>> n = Element("p")
    >>> from helpers import *
    >>> p = lmx(n)
    >>> isinstance(p.text("essai"), lmx)
    True
    >>> n.toxml() == '<p>essai</p>'
    True
    >>>
    """

def doctest_lmx_from_string():
    """
    >>> from helpers import *
    >>> p = lmx("p")
    >>> isinstance(p, lmx)
    True
    >>> isinstance(p.node, Element)
    True
    """

def doctest_text_node():
    """
    >>> p = lmx("p")
    >>> isinstance(p.text("foo"), lmx)
    True
    >>> p.node.toxml()
    '<p>foo</p>'
    >>>
    """

def doctest_add():
    """
    >>> from helpers import *
    >>> p = lmx("p")
    >>> isinstance(p.add("bar", c="y"), lmx)
    True
    >>> p.node.toxml() == '<p><bar c="y"/></p>'
    True
    >>>
    """

def doctest_lmx_abomination():
    """
    >>> from helpers import *
    >>> p = lmx("p")
    >>> s = p.select(name='category', selected='2')
    >>> isinstance(s, lmx)
    True
    >>> s.node.toxml() == '<select name="category" selected="2"/>'
    True
    >>> p.node.toxml() == '<p><select name="category" selected="2"/></p>'
    True
    >>> isinstance (p.b(a="c"), lmx)
    True
    >>> p.node.toxml() == '<p><select name="category" selected="2"/><b a="c"/></p>'
    True
    >>> p.b()["z"]="x"
    >>> p.node.toxml() == '<p><select name="category" selected="2"/><b a="c"/><b z="x"/></p>'
    True
    >>>
    """

def test_suite():
    import sys
    import doctest  
    return doctest.DocTestSuite(sys.modules[__name__])

if __name__ == "__main__":
    import unittest
    unittest.main(defaultTest='test_suite')

