# -*- coding: iso-8859-1 -*-
"""
Doctest etablissement

"""
def doctest_etab_add_connexion():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> isinstance(s.etabs, list)
    True
    >>> etab = s.etabs[0]
    >>> etab.get_id() == u'EZTab'
    True
    >>>
    >>> len(etab.connexions) == 2
    True
    >>> d =  {'nom':"test_nom", 'left':"test_left", 'left_nat':"test_leftnat", 'leftsubnet':"test_leftsubnet",'leftnexthop':"test_leftnexthop", "reseau":'sphynx-1'}
    >>> etab.add_connexion(d)
    u'EZTab-2'
    >>> len(etab.connexions) == 3
    True
    >>>
    """
def doctest_etab_add_connexion_without_left():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> d =  {'nom':"test_nom", 'left_nat':"test_leftnat", 'leftsubnet':"test_leftsubnet",'leftnexthop':"test_leftnexthop", "reseau":'sphynx-1'}
    >>> etab = s.etabs[0]
    >>> etab.add_connexion(d)
    u'EZTab-2'
    >>> len(etab.connexions) == 3
    True
    """

def doctest_etab_add_connexion_without_left_left_nat():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> d =  {'nom':"test_nom", 'leftsubnet':"test_leftsubnet",'leftnexthop':"test_leftnexthop", "reseau":'sphynx-1'}
    >>> etab = s.etabs[0]
    >>> etab.add_connexion(d)
    u'EZTab-2'
    >>> len(etab.connexions) == 3
    True
    """
def doctest_etab_add_connexion_without_left_left_nat_left_subnet():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> d =  {'nom':"test_nom", 'leftnexthop':"test_leftnexthop", "reseau":'sphynx-1'}
    >>> etab = s.etabs[0]
    >>> etab.add_connexion(d)
    u'EZTab-2'
    >>> len(etab.connexions) == 3
    True
    """
def doctest_etab_add_connexion_without_left_left_nat_left_subnet_nexthop():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> d =  {'nom':"test_nom", "reseau":'sphynx-1'}
    >>> etab = s.etabs[0]
    >>> etab.add_connexion(d)
    u'EZTab-2'
    >>> len(etab.connexions) == 3
    True
    """
def doctest_etab_add_connexion_nom():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> d =  {'nom':"test_nom"}
    >>> etab = s.etabs[0]
    >>> etab.add_connexion(d)
    u'EZTab-2'
    >>> len(etab.connexions) == 3
    True
    """
def doctest_etab_add_connexion_minimal():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> d =  {}
    >>> etab = s.etabs[0]
    >>> etab.add_connexion(d)
    u'EZTab-2'
    >>> len(etab.connexions) == 3
    True
    """
def doctest_etab_del_connexion():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> isinstance(s.etabs, list)
    True
    >>> etab = s.etabs[0]
    >>> etab.get_id() == u'EZTab'
    True
    >>>
    >>> len(etab.connexions)
    2
    >>> con2 = etab.connexions[1]
    >>> etab.del_connexion(con2.get_id())
    >>> len(etab.connexions)
    1
    >>> 
    """
def doctest_connexion():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> isinstance(s.etabs[0],Etab)
    True
    >>> etab = s.etabs[0]
    >>> conn = etab.connexions[0]
    >>> isinstance(conn, Connexion)
    True
    """

def doctest_add_translation():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> etab = s.etabs[0]
    >>> con = etab.connexions[0]
    >>> con.add_translation(source="essai",destination="essai_dest")
    >>> con.translations == [{'source': u'sdfsdf', 'destination': u'sdf'}, {'source': u'sdfsdf', 'destination': u'sdfsdf'}, {'source': 'essai', 'destination': 'essai_dest'}]
    True
    >>>
    """
def doctest_del_translation():
    """
    >>> from tunnel.entities import *
    >>> s = Sphynx("sphynx.xml")
    >>> etab = s.etabs[0]
    >>> con = etab.connexions[0]
    >>> con.add_translation(source="essai",destination="essai_dest")
    >>> len(con.translations) == 3
    True
    >>> con.del_translation(source="essai",destination="essai_dest")
    >>> len(con.translations) == 2
    True
    >>>
    """
def test_suite():
    import sys
    import doctest  
    return doctest.DocTestSuite(sys.modules[__name__])

if __name__ == "__main__":
    import unittest
    unittest.main(defaultTest='test_suite')
