#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""

Script de mise à jour des fichiers ipsec côté amon
(action directe sur les scripts)

"""

# configuration
# ipsec_conf = "ipsec.conf_Sphynx"
ipsec_conf = "/etc/freeswan/ipsec.conf"
ipsec_secret = "/etc/freeswan/ipsec.secrets"

# ne pas changer à partir d'ici
from os import system as run
import shutil

# utilitaires de manipulation de fichier
def make_backup(filepath):
    """fait une sauvegarde d'un fichier
    @param filepath : nom long du fichier
    """
    backup_file = filepath + '.bak'
    # print "%s --> %s" % (filepath, backup_file)
    shutil.copy(filepath, backup_file)

def insert_firstline(filepath, line):
    """insertion d'une ligne en début de fichier
    """
    # 1 - retrieve original content
    f = file(filepath)
    content = f.read()
    f.close()
    # 1bis. vérifions avant toute chose que la ligne n'est pas déjà insérée
    if not grep(filepath, line):
        # 2 - add coding line
        f = file(filepath, 'w')
        f.write(line + "\n" + content)
        f.close()

def insert_line(filepath, match, line, before=False):
    """insertion d'une ligne dans un fichier
    avant ou après la ligne 'line' donnée
    @param match : le motif dans le fichier là où il y aura une insertion
    @param line : la ligne à insérer
    @param before : booléen, pour insérer avant ou après 
    """
    # 1 - retrieve original content
    f = file(filepath)
    content = f.readlines()
    f.close()
    # 1bis. vérifions avant toute chose que la ligne n'est pas déjà insérée
    if not grep(filepath, line):
        # 2. suppression de la ligne dans le vecteur
        for i in content:
            if match in i:
                num = content.index(i)
        # avant ou après la ligne de repère
        if before : 
            content.insert(num, line+'\n')
        else:
            content.insert(num+1, line+'\n')
        
        # 3. ecriture dans le fichier
        fh = file(filepath, 'w')
        fh.write("".join(content))
        fh.close()
        
def delete_line(filepath, line):
    """
    Supprime la ligne 'line' dans le fichier 'filepath'
    """
    # 1. mettons le contenu du fichier dans un vecteur
    fh = file(filepath, 'r')
    content = fh.readlines()
    fh.close()
    # 2. suppression de la ligne dans le vecteur
    for l in content:
        if line in l:
            content.remove(l)
    # 3. ecriture dans le fichier
    fh = file(filepath, 'w')
    fh.write("".join(content))
    fh.close()
    
def grep(filepath, match):
    """
    @param filepath : le nom du fichier à explorer
    @param match : le motif à rechercher
    @return : True si ça matche, False sinon
    """
    fh = file(filepath, 'r')
    content = fh.read()
    fh.close()
    # vérifions que c'est bien un fichier généré par le sphynx
    if match in content:
        return True
    else:
        return False


# traitements des fichiers proprement dit
def update_ipsec_conf(filepath):
    """
    modifie le fichier ipsec.conf
    @param filnename : le fichier ipsec.conf
    
    """
    # est-ce qu'il s'agit d'un fichier généré par le sphynx 
    if not grep(filepath, "#DEB"):
        return 
    # fixons les droits en écriture
    cmd = "chmod +w %s " % filepath
    run(cmd)
    # copie de sauvegarde du fichier ipsec.conf en ipsec.conf.bak
    make_backup(filepath)
    # modification de la première ligne du fichier
    if not grep(filepath, "version 2"):
        insert_firstline(filepath, "version 2")
    # suppression de lignes
    delete_line(filepath, 'plutoload=%search')
    delete_line(filepath, 'plutostart=%search')
    # ajout de lignes
    if not grep(filepath, "ike=aes128-sha2"):
        insert_line(filepath, "pfs=yes", """        ike=aes128-sha2_256-modp2048,3des-md5-modp1536,3des-md5-modp1024,3des-sha-modp1536,3des-sha-modp1024
        esp=aes128-sha2_256,3des-md5,3des-sha1
        pfsgroup=modp2048,modp1536,modp1024""", before=False)
        
    if not grep (filepath, "rightfirewall=yes"):
        insert_line(filepath, "rightnexthop=", """        rightfirewall=yes
        righthostaccess=yes""", before=False)

    if not grep(filepath, "leftrsasigkey=%cert"):
        insert_line(filepath, "uniqueids=yes", "        nocrsend=yes")
    # remettons les droits en lecture seule
    cmd = "chmod -w %s " % filepath
    run(cmd)
	

# traitement fichier ipsec.secret
#def check_ipsec_secret():
    # on verifie si le fichier existe
#    fh = file(filepath, 'r')
#    content = fh.read()
#    fh.close()
    # on verifie s'il s'agit bien d'un fichier géneré par sphynx
#    if not grep(filepath, "#DEB")
        # on cherche dans d'autre fichier ipsec.secrets
#	os.sys
	# si on trouve la chaine dans un autre fichier, on le copie sous le nom de ipsec.secrets
	
def main():
    update_ipsec_conf(ipsec_conf)
#    check_ipsec_secret(ipsec_secrets)
if __name__ == "__main__":
    main()
